﻿using Bots.Grind;
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.CommonBot.Coroutines;
using Styx.Pathing;
using Styx.Plugins;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Styx.TreeSharp;
using Buddy.Coroutines;
using CommonBehaviors.Actions;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LogoutHelper
{
    public class LogoutHelper : HBPlugin
    {
        public override string Author { get { return "Opply"; } }
        public override string Name { get { return "Logout Helper"; } }
        public override Version Version { get { return new Version(1, 0); } }
        public override string ButtonText { get { return "Settings"; } }
        public override bool WantButton { get { return true; } }

        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private DateTime useItemTime;
        private DateTime logoutTime;
        private DateTime killWoWTime;

		private Composite logoutComposite;
		private bool isLoggingOut = false;
		
        public LogoutHelper()
        {

        }

        public override void OnButtonPress()
        {
            SettingsForm form = new SettingsForm();
            form.ShowDialog();
            Settings.Instance.Save();

            setLogoutTime();
        }

        public override void OnEnable()
        {
            Log("Enabled");
            base.OnEnable();
            BotEvents.OnBotStarted += OnBotStarted;

            setLogoutTime();
        }

        public override void OnDisable()
        {
            Log("Disabled");
            BotEvents.OnBotStopped += OnBotStopped;
			
			if(isLoggingOut)
			{
				TreeHooks.Instance.RemoveHook("Combat_OOC", logoutComposite);
				isLoggingOut = false;
			}
			
            base.OnDisable();
        }

        public void OnBotStarted(EventArgs args)
        {
            setLogoutTime();
        }

        public void OnBotStopped(EventArgs args)
        {
        }

        private void setLogoutTime()
        {
            if (Styx.CommonBot.TreeRoot.IsRunning)
            {
                logoutTime = DateTime.Now.AddSeconds(Settings.Instance.SecondsToLogout);
                Log(string.Format($"Logging out in { Settings.Instance.SecondsToLogout } seconds"));
            }
        }

        public override void Pulse()
        {
            if (Me.Combat || !Me.IsAlive || Me.IsGhost || Me.IsOnTransport || Me.OnTaxi || Me.Stunned)
            {
                return;
            }

            if (DateTime.Now > logoutTime)
            {
				if(!isLoggingOut)
				{
					isLoggingOut = true;
					Log(string.Format("Attaching logout treehook."));
					TreeHooks.Instance.InsertHook("Combat_OOC", 0, logoutComposite ?? (logoutComposite = new ActionRunCoroutine(ctx => logoutCoroutine())));
				}
            }
        }

		private async Task<bool> logoutCoroutine()
		{
			if ( Me.Combat || !Me.IsAlive || Me.IsGhost || Me.IsOnTransport || Me.OnTaxi || Me.Stunned)
            {
                return false;
            }
			
			await CommonCoroutines.LandAndDismount(null, true);
			await CommonCoroutines.StopMoving();
			
			if( !await useItem( Settings.Instance.ItemToUseBeforeLogout ) )
			{
				Log(string.Format("Failed to use item, logging out."));
				await logout();
				return true;
			}
			
			await Coroutine.Sleep(1000);
			await Coroutine.Wait(10000, () => !Me.IsCasting);
			
			await logout();
			
			return true;
		}
		
        private async Task<bool> useItem(int itemId)
        {
            WoWItem item = Me.BagItems.FirstOrDefault(i => i.ItemInfo.Id == itemId);

            if (item == null)
            {
                Log(string.Format($"Item with id { itemId } not found"));
                return false;
            }
            if (!item.Usable)
            {
                Log(string.Format($"Item { item.Name } not usable"));
                return false;
            }
            if (item.Cooldown > 0)
            {
                Log(string.Format($"Item { item.Name } on cd"));
                return false;
            }

            Log(string.Format($"Using item {item.Name}"));
            return item.Use();
        }

        public async Task<bool> logout()
        {
			await Coroutine.Sleep(10000);
			
			Log("Logging out...");
            Lua.DoString("ForceQuit()");
			
			await Coroutine.Sleep(10000);
			killWow();
			TreeRoot.Stop("LogoutHelper logging out.");
			return true;
        }

        public void killWow()
        {
            int wowProcessId = StyxWoW.Memory.Process.Id;
            if (Process.GetProcessById(wowProcessId) != null)
            {
				Log("Killing wow");
                Process.GetProcessById(wowProcessId).Kill();
            }
        }

        public void Log(string text)
        {
            Logging.Write(String.Format("LogoutHelper {0}: {1}", Version, text));
        }
    }
}
